/**
 * BulkUploadSmttFilesWorker.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.client.worker;

import java.util.ArrayList;
import java.util.List;

import com.smartmatic.waffle.ecm.client.ECMServiceProvider;
import com.smartmatic.waffle.ecm.client.lang.ConnectionException;
import com.smartmatic.waffle.ecm.client.ui.common.AsynchronousOperation;
import com.smartmatic.waffle.ecm.client.ui.common.AwtOperationType;
import com.smartmatic.waffle.ecm.data.transfer.objects.BulkResultDTO;
import com.smartmatic.waffle.ecm.data.transfer.objects.FileToUploadDTO;
import com.smartmatic.waffle.ecm.data.transfer.objects.UploadedFileDetailDTO;
import com.smartmatic.waffle.ecm.data.transfer.type.SmttFileType;
import com.smartmatic.waffle.ecm.service.lang.BulkDataLoadException;
import com.smartmatic.waffle.ecm.service.lang.BulkDataLoadException.BulkDataLoadExceptionOperationExceptionInfo;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * Class that executes the main process to execute bulk upload from smartmatic
 * files.
 */
public class BulkUploadSmttFilesWorker extends AsynchronousOperation {

	public BulkDataLoadExceptionOperationExceptionInfo bulkLoadExceptionInfo;

	/**
	 * Gets the list of smtt files to upĺoad.
	 * 
	 * @return the list of smtt files to upĺoad
	 */
	public List<FileToUploadDTO> getSmttFilesToUpĺoadList() {
		return this.smttFilesToUpĺoadList;
	}

	/**
	 * BulkUploadSmttFilesWorker Constructor.
	 * 
	 * @param smttFilesToUpĺoadList
	 *            Input list with files to load
	 * @param resourceFileContent
	 *            Hold the byte array with the resources file
	 */
	public BulkUploadSmttFilesWorker(
			List<FileToUploadDTO> smttFilesToUpĺoadList,
			byte[] resourceFileContent) {
		this.smttFilesToUpĺoadList = smttFilesToUpĺoadList;
		this.resourceFileContent = resourceFileContent;
		this.fileUploadedDetailList = new ArrayList<UploadedFileDetailDTO>();
	}

	/**
	 * Gets the number of processed files.
	 * 
	 * @return the number of processed files
	 */
	public int getNumberOfProcessedFiles() {
		return this.numberOfProcessedFiles;
	}

	/**
	 * Gets the file being processed.
	 * 
	 * @return the file being processed
	 */
	public String getFileBeingProcessed() {
		return this.fileBeingProcessed;
	}

	/**
	 * Gets the uploaded file's details list.
	 * 
	 * @return the file uploaded file's details list
	 */
	public List<UploadedFileDetailDTO> getFileUploadedDetailList() {
		return this.fileUploadedDetailList;
	}

	/**
	 * Method that performs the bulk upload of smartmatic files.
	 */
	public void startBulkUploadOperation() {
		try {
			for (FileToUploadDTO fileToUpload : this.smttFilesToUpĺoadList) {

				logger.info("Processing "
						+ fileToUpload.getFileToUploadDetailDTO().getFileInfo()
								.getName() + " file");

				UploadedFileDetailDTO fileUploadedDetailDTO = new UploadedFileDetailDTO();

				this.fileBeingProcessed = fileToUpload
						.getFileToUploadDetailDTO().getFileInfo().getName();

				fireAwtEvent(AwtOperationType.UPDATE_COMPONENTS);

				String fileToUploadName = fileToUpload
						.getFileToUploadDetailDTO().getFileInfo().getName();

				fileUploadedDetailDTO.setFileName(fileToUploadName);

				byte[] fileToUploadContent = fileToUpload
						.getFileToUploadDetailDTO().getFileContent();

				SmttFileType smttFileType = fileToUpload.getSmttFileType();

				fileUploadedDetailDTO.setSmttFileType(smttFileType);

				BulkResultDTO bulkResultDTO = uploadSmttFileIntoDataBase(
						fileToUploadName, fileToUploadContent, smttFileType,
						this.resourceFileContent);

				fileUploadedDetailDTO.setBulkResultDTO(bulkResultDTO);

				this.fileUploadedDetailList.add(fileUploadedDetailDTO);

				if (bulkResultDTO.getIncorrectCount() > 0) {
					break;
				}

				checkConfiguredRegionLaguages();

				this.numberOfProcessedFiles++;

				fireAwtEvent(AwtOperationType.UPDATE_PROGRESS_BAR);
			}

			fireAwtEvent(AwtOperationType.SHOW_RESULT_MESSAGE);
		} catch (BulkDataLoadException e) {

			logger.error(e, "Error loading Smtt files");
			this.numberOfProcessedFiles = 0;
			this.fileUploadedDetailList.clear();

			if (e.getInfo()
					.equals(BulkDataLoadExceptionOperationExceptionInfo.BADLY_CONFIGURED_AUDIO_VOTING_FOR_REGION)) {
				fireAwtEvent(AwtOperationType.REPORT_ERROR_IN_PROCESS);
			} else {
				this.bulkLoadExceptionInfo = (BulkDataLoadExceptionOperationExceptionInfo) e
						.getInfo();
				fireAwtEvent(AwtOperationType.SHOW_ERROR_MESSAGE);
			}

		} catch (Exception e) {
			logger.error(e, "Unexpected error loading Smtt file");

			this.numberOfProcessedFiles = 0;
			this.fileUploadedDetailList.clear();
			this.bulkLoadExceptionInfo = BulkDataLoadExceptionOperationExceptionInfo.UNEXPECTED_ERROR_EXECUTING_BULK_DATA_LOAD;
			fireAwtEvent(AwtOperationType.SHOW_ERROR_MESSAGE);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.waffle.ecm.client.ui.common.AsynchronousOperation#
	 * performOperation()
	 */
	@Override
	protected void performOperation() {
		startBulkUploadOperation();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.smartmatic.waffle.ecm.client.ui.common.AsynchronousOperation#onCompleted
	 * ()
	 */
	@Override
	protected void onCompleted() {
	}

	/**
	 * Method that calls the process in server side to execute bulk upload of
	 * smartmatic files
	 * 
	 * @param fileToUploadName
	 *            name of the file to upload
	 * @param fileToUploadContent
	 *            content of the file to upload
	 * @param fileType
	 *            type of file
	 * @param resourceFileContent
	 *            content of the resource file
	 * 
	 * @return dto with the result of bulk upload
	 * 
	 * @throws BulkDataLoadException
	 *             signal that an exception occurred executing bulk upload
	 * 
	 * @throws ConnectionException
	 *             Signal that an error occurred with the connection to the
	 *             server
	 */
	private BulkResultDTO uploadSmttFileIntoDataBase(String fileToUploadName,
			byte[] fileToUploadContent, SmttFileType fileType,
			byte[] resourceFileContent) throws BulkDataLoadException,
			ConnectionException {

		return ECMServiceProvider.uploadSmttFileIntoDataBase(fileToUploadName,
				fileToUploadContent, fileType, resourceFileContent);
	}

	/**
	 * Validate that the language of the regions with audio voting enabled is
	 * configured correctly.
	 * 
	 * @throws BulkDataLoadException
	 */
	private void checkConfiguredRegionLaguages() throws BulkDataLoadException {
		ECMServiceProvider.checkConfiguredRegionLaguages();
	}

	/**
	 * Static reference to log any info or error which occurred executing
	 * methods from this class.
	 */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(BulkUploadSmttFilesWorker.class);

	/** Input list with files to load. */
	private List<FileToUploadDTO> smttFilesToUpĺoadList;

	/** Holds the byte array with the resources file. */
	private byte[] resourceFileContent;

	/** Holds the amount of processed files to show the process status. */
	private int numberOfProcessedFiles = 0;

	/**
	 * Holds the information about the file being processing in the moment to
	 * show the process status.
	 */
	private String fileBeingProcessed;

	/** Output list with the detail of elements uploaded. */
	private List<UploadedFileDetailDTO> fileUploadedDetailList;
}
