/**
 * @file	CustomAttributeSelectionBoxWidget.cxx
 * @brief	Body custom attribute selection box widget class.
 * @author	Yadickson Soto
 * @date	13/11/2012
 */

#include "CustomAttributeSelectionBoxWidget.hxx"
#include "Configuration/s-a-e-s-g-u-i-conf-manager.h"
#include "Resources/election-string-resources-manager.h"
#include <Operation/Voting/VotingOperation.hxx>
#include "FactoryVotingWidget.hxx"
#include <System/Utils/StringUtils.hxx>

#ifdef HAVE_CONFIG_H
#include "../../../config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Widgets::Voting;
using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::Voting::VotingDevice;
using namespace Smartmatic::SAES::Operation::Voting;
using namespace Smartmatic::SAES::Resources;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::GUI::Operation::Voting;
using namespace Smartmatic::System::Utils;

CustomAttributeSelectionBoxWidget::CustomAttributeSelectionBoxWidget ( Custom_attribute_definition & customAttr, bool showConfirmation, bool sortAlphabetically )
    : ICustomAttributeSelectionWidget ( customAttr, showConfirmation, sortAlphabetically )
{
    currentBox = NULL;
    headerLabel = NULL;
    bottomFrame = NULL;
    externalBox = NULL;
    tableWidget = NULL;
    buttonWidget = NULL;
    flagForce = false;

    initialize();

    setWidget ( externalBox );
    setAccessibleName ( "CustomAttributeSelectionBox" );
}

CustomAttributeSelectionBoxWidget::~CustomAttributeSelectionBoxWidget()
{
    remove();
    dispose();
}

void CustomAttributeSelectionBoxWidget::ForceSelectedOption ( unsigned int index )
{
    flagForce = true;
    tableWidget->forceClick ( index );
    buttonWidget->showButton ( true, true, false, false );
}

void CustomAttributeSelectionBoxWidget::initialize()
{
    int spacing = SAESGUIConfigurationManager::getInstance()->getCustomAttributeSelectionWidgetConfiguration().getBoxSpacing();

    tableWidget = manage ( new TableBoxSelectionWidget ( false, false, true ) );
    tableWidget->setSpacings ( spacing, 0 );

    buttonWidget = FactoryVotingWidget::createCustomAttributeSelectionButtonWidget();
    buttonWidget->update();

    bool enableClick = SAESGUIConfigurationManager::getInstance()->getCustomAttributeSelectionWidgetConfiguration().getEnableClick();

    if ( enableClick )
    {
        tableWidget->getSignalClick().connect ( sigc::mem_fun ( this, &CustomAttributeSelectionBoxWidget::onBoxClicked ) );
        buttonWidget->getSignalAcceptButtonClicked().connect ( sigc::mem_fun ( this, &CustomAttributeSelectionBoxWidget::onAcceptClicked ) );
        buttonWidget->getSignalClearButtonClicked().connect ( sigc::mem_fun ( this, &CustomAttributeSelectionBoxWidget::onClearClicked ) );
    }

    int headerHeight = SAESGUIConfigurationManager::getInstance()->getCustomAttributeSelectionWidgetConfiguration().getFrameTopHeight();
    int bottomHeight = SAESGUIConfigurationManager::getInstance()->getCustomAttributeSelectionWidgetConfiguration().getFrameBottomHeight();

    int boxWidth = SAESGUIConfigurationManager::getInstance()->getCustomAttributeSelectionWidgetConfiguration().getBoxWidth();
    int boxHeight = SAESGUIConfigurationManager::getInstance()->getCustomAttributeSelectionWidgetConfiguration().getBoxHeight();

    headerLabel = new SmartmaticBasicLabel();
    headerLabel->show();
    headerLabel->set_size_request ( -1, headerHeight );

    Smartmatic::GUI::Configuration::LabelConfiguration labelConf = SAESGUIConfigurationManager::ConvertToLabelConfiguration ( SAESGUIConfigurationManager::getInstance()->getCustomAttributeSelectionWidgetConfiguration().getHeaderLabelConfiguration() );
    Smartmatic::GUI::Configuration::LabelConfiguration::SetLabelConfiguration ( headerLabel, labelConf );

    bottomFrame = new SmartmaticBasicFrame();
    bottomFrame->set_shadow_type ( Gtk::SHADOW_NONE );
    bottomFrame->show();
    bottomFrame->add ( *buttonWidget );
    bottomFrame->set_size_request ( -1, bottomHeight );

    externalBox = new SmartmaticBasicVBox ( false , 0 );

    if ( headerHeight > 0 )
    {
        Gtk::HSeparator * separator = manage ( new Gtk::HSeparator() );
        externalBox->pack_start ( *headerLabel, false, false );
        externalBox->pack_start ( *separator, false, false );
    }

    externalBox->pack_start ( *tableWidget );
    externalBox->pack_end ( *bottomFrame, false, false, 5 );

    buttonWidget->showButton ( false, false, false, false );

    Gdk::Color selected = SAESGUIConfigurationManager::ConvertToGdkColor ( SAESGUIConfigurationManager::getInstance()->getCustomAttributeSelectionWidgetConfiguration().getSelectedColor() );
    Gdk::Color unselected = SAESGUIConfigurationManager::ConvertToGdkColor ( SAESGUIConfigurationManager::getInstance()->getCustomAttributeSelectionWidgetConfiguration().getUnSelectedColor() );
    Smartmatic::GUI::Configuration::LabelConfiguration boxLabelConf = SAESGUIConfigurationManager::ConvertToLabelConfiguration ( SAESGUIConfigurationManager::getInstance()->getCustomAttributeSelectionWidgetConfiguration().getBoxLabelConfiguration() );

    bool showBorderBox = SAESGUIConfigurationManager::getInstance()->getCustomAttributeSelectionWidgetConfiguration().getShowBorder();

    std::list < std::string >::iterator it;
    int pos = 0;

    sortCustomAttribute ();

    for ( it = cadList.begin();
            it != cadList.end();
            ++it, pos++ )
    {
        std::string name = ElectionStringResourcesManager::getInstance()->getResource ( *it );
        BoxSelectionWidget * box = manage ( new BoxSelectionWidget ( *it, enableClick, showBorderBox ) );
        box->setSelectedColor ( selected );
        box->setUnselectedColor ( unselected );
        box->set_size_request ( boxWidth, boxHeight );

        SmartmaticBasicLabel * label = manage ( new SmartmaticBasicLabel ( name ) );
        Smartmatic::GUI::Configuration::LabelConfiguration::SetLabelConfiguration ( label, boxLabelConf );
        label->set_size_request ( boxWidth, boxHeight );

        box->addWidget ( label );
        box->set_name ( "voting_experience_custom_attribute_widget" );
        box->setName ( "custom_attribute_widget_event_box" );

        std::string code = StringUtils::intToString ( pos + 1 );

        box->setAccessibleName ( "CustomAttributeBox" + code );
        label->setAccessibleName ( "CustomAttributeBox" + code );

        tableWidget->put ( box, pos, 0 );
    }

    tableWidget->setAccessibleName ( "CustomAttributeSelectionMainTable" );
    headerLabel->setAccessibleName ( "CustomAttributeSelectionTitle" );
    bottomFrame->setAccessibleName ( "CustomAttributeSelectionBottom" );
    externalBox->setAccessibleName ( "CustomAttributeSelectionContainer" );
}

void CustomAttributeSelectionBoxWidget::dispose()
{
    if ( headerLabel )
    {
        delete headerLabel;
        headerLabel = NULL;
    }

    if ( bottomFrame )
    {
        delete bottomFrame;
        bottomFrame = NULL;
    }

    if ( externalBox )
    {
        delete externalBox;
        externalBox = NULL;
    }

    if ( buttonWidget )
    {
        delete buttonWidget;
        buttonWidget = NULL;
    }
}

void CustomAttributeSelectionBoxWidget::refresh()
{
    buttonWidget->update();

    Glib::ustring title;
    Glib::ustring key = _ ( "Smartmatic.SAES.GUI.Widgets.Voting.CustomAttributeSelectionBoxWidget.Title" );

    std::string nameKey = ElectionStringResourcesManager::getInstance()->getResource ( customAttr.getName_key() );
    std::string descriptionKey = ElectionStringResourcesManager::getInstance()->getResource ( customAttr.getDescription_key() );

    title = Glib::ustring::compose ( key, nameKey, descriptionKey );

    headerLabel->set_text ( title );

    if ( !flagForce )
    {
        onClearClicked();
    }
}

void CustomAttributeSelectionBoxWidget::onAcceptClicked()
{
    if ( currentBox )
    {
        signal_Answer().emit ( currentBox->getCode() );
    }
}

void CustomAttributeSelectionBoxWidget::onClearClicked()
{
    buttonWidget->showButton ( false, false, false, false );
    tableWidget->clear();
    buttonWidget->clearSignals();
}

void CustomAttributeSelectionBoxWidget::onBoxClicked ( std::list < Smartmatic::GUI::Widgets::BoxSelectionWidget * > & list )
{
    bool selected = !list.empty();

    if ( selected )
    {
        currentBox = ( * ( list.begin() ) );
    }
    else
    {
        currentBox = NULL;
    }

    if ( showConfirmation )
    {
        buttonWidget->showButton ( selected, selected, false, false );
    }
    else if ( selected )
    {
        signal_Answer().emit ( currentBox->getCode() );
    }
}
