/**
 * @file CertUtil.cpp
 * @brief Body class utility certified manager
 * @date 10/11/2011
 * @author Freddy Gomez
 */

#include "CertUtil.h"
#include <System/Security/Cryptography/CertificateExtensionPEM.hxx>
#include <System/Security/Cryptography/CertificateExtensionDER.hxx>
#include <System/Security/Cryptography/CertificateExtensionPKCS12.hxx>
#include <System/IO/File.h>
#include <openssl/evp.h>
#include <openssl/x509.h>
#include <openssl/safestack.h>

using namespace Smartmatic::System::Security::Cryptography;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System;

X509 * CertUtil::LoadCertifiedX509 ( const std::string & certPath, CertUtil::CertFileFormat format, const std::string & certPass )
{
    SafeBuffer certificateBuffer;
    File::getInstance()->openFileToBuffer ( certPath, certificateBuffer );

    return LoadCertifiedX509 ( certificateBuffer, format, certPass );
}

EVP_PKEY * CertUtil::LoadCertifiedPrivateKey ( const std::string & certPath, CertUtil::CertFileFormat format, const std::string & certPass )
{
    SafeBuffer certificateBuffer;
    File::getInstance()->openFileToBuffer ( certPath, certificateBuffer );

    return LoadCertifiedPrivateKey ( certificateBuffer, format, certPass );
}

STACK_OF ( X509 ) * CertUtil::LoadCertificateAuthorities ( const std::string & certPath, CertUtil::CertFileFormat format, const std::string & certPass )
{
    SafeBuffer certificateBuffer;
    File::getInstance()->openFileToBuffer ( certPath, certificateBuffer );

    return LoadCertificateAuthorities ( certificateBuffer, format, certPass );
}

X509 * CertUtil::LoadCertifiedX509 ( Smartmatic::System::SafeBuffer & certificateBuffer, CertUtil::CertFileFormat format, const std::string & certPass )
{
    CertificateExtensionBase * fileExt = getCertificateExtensionBase ( certificateBuffer, format, certPass );

    X509 * certificate;
    certificate = fileExt->LoadCertifiedX509( );
    fileExt->releaseCertifiedPrivateKey();
    fileExt->releaseCertificateAuthorities();
    delete fileExt;

    return certificate;
}

EVP_PKEY * CertUtil::LoadCertifiedPrivateKey ( Smartmatic::System::SafeBuffer & certificateBuffer, CertUtil::CertFileFormat format, const std::string & certPass )
{
    CertificateExtensionBase * fileExt = getCertificateExtensionBase ( certificateBuffer, format, certPass );

    EVP_PKEY * certificate;
    certificate = fileExt->LoadCertifiedPrivateKey( );
    fileExt->releaseCertifiedX509();
    fileExt->releaseCertificateAuthorities();
    delete fileExt;

    return certificate;
}

STACK_OF ( X509 ) * CertUtil::LoadCertificateAuthorities ( Smartmatic::System::SafeBuffer & certificateBuffer, CertUtil::CertFileFormat format, const std::string & certPass )
{
    CertificateExtensionBase * fileExt = getCertificateExtensionBase ( certificateBuffer, format, certPass );

    STACK_OF ( X509 ) * certificateAuthorities;
    certificateAuthorities = fileExt->getCertificateAuthorities ( );
    fileExt->releaseCertifiedX509();
    fileExt->releaseCertifiedPrivateKey();
    delete fileExt;

    return certificateAuthorities;
}

CertificateExtensionBase * CertUtil::getCertificateExtensionBase ( const std::string & certPath, CertFileFormat format, const std::string & certPass )
{
    SafeBuffer certificateBuffer;
    File::getInstance()->openFileToBuffer ( certPath, certificateBuffer );

    return getCertificateExtensionBase ( certificateBuffer, format, certPass );
}

CertificateExtensionBase * CertUtil::getCertificateExtensionBase ( Smartmatic::System::SafeBuffer & certificate, CertUtil::CertFileFormat format, const std::string & certPass )
{
    CertificateExtensionBase * certificateBase = NULL;

    switch ( format )
    {
        case CertUtil::CertFormat_PEM:
            certificateBase = new CertificateExtensionPEM ( certificate, certPass );
            break;

        case CertUtil::CertFormat_PKCS12:
            certificateBase = new CertificateExtensionPKCS12 ( certificate, certPass );
            break;

        case CertUtil::CertFormat_DER:
        default:
            certificateBase = new CertificateExtensionDER ( certificate );
            break;
    }

    return certificateBase;
}
