/**
 * @file CertUtil.h
 * @brief Header class utility certified manager
 * @date 10/11/2011
 * @author Freddy Gomez
 */

#ifndef CERTUTIL_H_
#define CERTUTIL_H_

#include <string>
#include <System/SafeBuffer.h>
#include <System/Security/Cryptography/CertificateExtensionBase.hxx>

namespace Smartmatic
{
    namespace System
    {
        namespace Security
        {
            namespace Cryptography
            {
                /**
                 * @class	CertUtil
                 * @brief	Class utility certified manager.
                 * @author	Freddy Gomez
                 * @date	10/11/2011
                 */
                class CertUtil
                {

                    public:

                        /**
                         * @enum CertFileFormat
                         * @brief	enum format certified.
                         * ### date	10/11/2011.
                         * ### author	Freddy Gomez.
                         */
                        typedef enum
                        {
                            CertFormat_DER      = 1,		///< ANS1
                            CertFormat_PEM      = 3,		///< pem
                            CertFormat_PKCS12   = 5		///< pkcs12
                        } CertFileFormat;

                        /**
                         * @fn	static EVP_PKEY * LoadCertifiedPrivateKey ( const std::string & certPath, CertFileFormat format, const std::string & certPass = "" );
                         * @brief	Method load private key certified from file path.
                         * @author	Yadickson Soto
                         * @date	21/03/2012
                         * @param	certPath	path file.
                         * @param	format		format certified.
                         * @param	certPass	password.
                         * @return	private key.
                         */
                        static EVP_PKEY * LoadCertifiedPrivateKey ( const std::string & certPath, CertFileFormat format, const std::string & certPass = "" );

                        /**
                         * @fn	static X509 * LoadCertifiedX509 ( const std::string & certPath, CertFileFormat format, const std::string & certPass = "" );
                         * @brief	Method load certified file.
                         * @author	Yadickson Soto
                         * @date	10/11/2011
                         * @param	certPath	path file.
                         * @param	format		format certified.
                         * @param	certPass	password.
                         * @return	certificate.
                         */
                        static X509 * LoadCertifiedX509 ( const std::string & certPath, CertFileFormat format, const std::string & certPass = "" );

                        /**
                         * @fn	static stack_st_X509 * LoadCertificateAuthorities ( const std::string & certPath, CertFileFormat format, const std::string & certPass = "" );
                         * @brief	Method load certificate from SafeBuffer.
                         * @author	Beatriz Avila
                         * @date	18/12/2013
                         * @param	certificate	safe buffer with certificate.
                         * @param	format		format certified.
                         * @param	certPass	password.
                         * @return	certificate.
                         */
                        static stack_st_X509 * LoadCertificateAuthorities ( const std::string & certPath, CertFileFormat format, const std::string & certPass = "" );

                        /**
                         * @fn	static EVP_PKEY * LoadCertifiedPrivateKey ( Smartmatic::System::SafeBuffer & certificate, CertFileFormat format, const std::string & certPass = "" );
                         * @brief	Method load certified private key from SafeBuffer.
                         * @author	Beatriz Avila
                         * @date	18/12/2013
                         * @param	certificate	safe buffer with certificate.
                         * @param	format		format certified.
                         * @param	certPass	password.
                         * @return	private key.
                         */
                        static EVP_PKEY * LoadCertifiedPrivateKey ( Smartmatic::System::SafeBuffer & certificate, CertFileFormat format, const std::string & certPass = "" );

                        /**
                         * @fn	static X509 * LoadCertifiedX509 ( Smartmatic::System::SafeBuffer & certificate, CertFileFormat format, const std::string & certPass = "" );
                         * @brief	Method load certificate from SafeBuffer.
                         * @author	Beatriz Avila
                         * @date	18/12/2013
                         * @param	certificate	safe buffer with certificate.
                         * @param	format		format certified.
                         * @param	certPass	password.
                         * @return	certificate.
                         */
                        static X509 * LoadCertifiedX509 ( Smartmatic::System::SafeBuffer & certificate, CertFileFormat format, const std::string & certPass = "" );

                        /**
                         * @fn	static stack_st_X509 * LoadCertificateAuthorities ( Smartmatic::System::SafeBuffer & certificate, CertFileFormat format, const std::string & certPass = "" );
                         * @brief	Method load certificate from SafeBuffer.
                         * @author	Beatriz Avila
                         * @date	18/12/2013
                         * @param	certificate	safe buffer with certificate.
                         * @param	format		format certified.
                         * @param	certPass	password.
                         * @return	certificate.
                         */
                        static stack_st_X509 * LoadCertificateAuthorities ( Smartmatic::System::SafeBuffer & certificate, CertFileFormat format, const std::string & certPass = "" );

                        /**
                         * @fn	static CertificateExtensionBase * getCertificateExtensionBase ( const std::string & certPath, CertFileFormat format, const std::string & certPass = "" );
                         * @brief	Method load certificate extension base object.
                         * @author	Yadickson Soto
                         * @date	09/01/2014
                         * @param	certPath	path file.
                         * @param	format		format certified.
                         * @param	certPass	password.
                         * @return	certificate extension base.
                         */
                        static CertificateExtensionBase * getCertificateExtensionBase ( const std::string & certPath, CertFileFormat format, const std::string & certPass = "" );

                        /**
                         * @fn	static CertificateExtensionBase * getCertificateExtensionBase ( Smartmatic::System::SafeBuffer & certificate, CertFileFormat format, const std::string & certPass = "" );
                         * @brief	Method load certificate extension base object.
                         * @author	Yadickson Soto
                         * @date	09/01/2014
                         * @param	certificate	safe buffer with certificate.
                         * @param	format		format certified.
                         * @param	certPass	password.
                         * @return	certificate extension base.
                         */
                        static CertificateExtensionBase * getCertificateExtensionBase ( Smartmatic::System::SafeBuffer & certificate, CertFileFormat format, const std::string & certPass = "" );
                };
            } /* namespace Cryptography */
        } /* namespace Security */
    } /* namespace System */
} /* namespace Smartmatic */
#endif /* CERTUTIL_H_ */
