/**
 * @file MsgDigestUtil.h
 * @brief Header message digest utility class
 * @date 01/04/2012
 * @author Marcel Cordovi
 */

#ifndef MSGDIGESTUTIL_H_
#define MSGDIGESTUTIL_H_


#include <string>
#include "System/Exception/MessageDigestException.h"
#include "System/SafeBuffer.h"

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace System
    {
        namespace Security
        {
            namespace Cryptography
            {
                /**
                 * @class	MsgDigestUtil
                 * @brief	Message digest utility class.
                 * @author	Yadickson Soto
                 * @date	29/04/2011
                 */
                class MsgDigestUtil
                {
                    public:

                        /**
                         * @enum	DigestAlgType
                         * @brief	Values that represent DigestAlgType.
                         */
                        enum DigestAlgType
                        {
                            DGSTUTIL_MD5,
                            DGSTUTIL_SHA1,
                            DGSTUTIL_SHA224,
                            DGSTUTIL_SHA256,
                            DGSTUTIL_SHA384,
                            DGSTUTIL_SHA512
                        };

                        /**
                         * @brief  Returns the default algorithm for hashing files.
                         * @author manuel.arguelles
                         * @date   Jul 18, 2017
                         * @return The digest algorithm to use.
                         */
                        static DigestAlgType getDefaultFileDigest();

                        /**
                         * @brief  Returns the default algorithm for hashing file names.
                         * @author manuel.arguelles
                         * @date   Jul 18, 2017
                         * @return The digest algorithm to use.
                         */
                        static DigestAlgType getDefaultFileNameDigest();

                        /**
                         * @brief  Returns the default hashing algorithm for certificates.
                         * @author manuel.arguelles
                         * @date   Jul 18, 2017
                         * @return The digest algorithm to use.
                         */
                        static DigestAlgType getDefaultCertificateDigest();

                        /**
                         * @brief  Return the default hashing algorithm for opening code computation.
                         * @author manuel.arguelles
                         * @date   Jul 18, 2017
                         * @return The digest algorithm to use.
                         */
                        static DigestAlgType getDefaultOpeningCodeDigest();

                        /**
                         * @brief  Return the default hashing algorithm for secret key signing.
                         * @author manuel.arguelles
                         * @date   Jul 18, 2017
                         * @return The digest algorithm to use.
                         */
                        static DigestAlgType getDefaultSecretKeyDigest();

                        /**
                         * @brief  Return the default hashing algorithm for compose key computation.
                         * @author manuel.arguelles
                         * @date   Jul 18, 2017
                         * @return The digest algorithm to use.
                         */
                        static DigestAlgType getDefaultComposeKeyDigest();

                        /**
                         * @brief  Return the default hashing algorithm used for (password based) key derivations.
                         * @author manuel.arguelles
                         * @date   Jul 19, 2017
                         * @return The digest algorithm to use.
                         */
                        static DigestAlgType getDefaultGenerateKeyDigest();

                        /**
                         * @brief  Return the default hashing algorithm to use for vote checksum.
                         * @author manuel.arguelles
                         * @date   Jul 20, 2017
                         * @return The digest algorithm to use.
                         */
                        static DigestAlgType getDefaultVoteDigest();

                        /**
                         * @fn	MsgDigestUtil( const Smartmatic::System::SafeBuffer & buffer );
                         * @brief	Default constructor.
                         * @author	Yadickson Soto
                         * @date	29/10/2012
                         * @param buffer data buffer
                         */
                        explicit MsgDigestUtil ( const Smartmatic::System::SafeBuffer & buffer );

                        /**
                         * @fn	virtual MsgDigestUtil::~MsgDigestUtil();
                         * @brief	Destructor.
                         * @author	Youssef.tourkmani
                         * @date	29/10/2012
                         */
                        virtual ~MsgDigestUtil();

                        /**
                         * @fn	void getDigest ( Smartmatic::System::SafeBuffer & result,
                         *   DigestAlgType alg = DGSTUTIL_SHA256)
                         * throw ( Smartmatic::System::Exception::MessageDigestException );
                         * @brief	Calculates hash value of a given binary message with the specified algorithm.
                         * @author	Yadickson Soto
                         * @date	05/10/2012
                         * @param	result	 	The message to digest result.
                         * @param	alg		 	algorithm type.
                         */
                        void getDigest ( Smartmatic::System::SafeBuffer & result, DigestAlgType alg)
                        throw ( Smartmatic::System::Exception::MessageDigestException );

                    protected:

                        /**
                         * @fn	virtual const char* MsgDigestUtil::mMapAlgName (DigestAlgType alg);
                         * @brief	Map algorithm name.
                         * @author	marcel
                         * @date	28/04/2011
                         * @param	alg	The algorithm.
                         * @return	null if it fails, else.
                         */

                        virtual const char * mMapAlgName ( DigestAlgType alg );

                        /**
                         * @fn	unsigned char* MsgDigestUtil::mPerformHashing ( const char* algName ,
                         * 			const unsigned char* data, int dataLen , int& outBufSize);
                         * @brief	Perform hashing.
                         * @author	marcel
                         * @date	28/04/2011
                         * @param	algName			Name of the algorithm.
                         * @param	data				The data.
                         * @param	dataLen			Length of the data.
                         * @param	outBufSize	Size of the out buffer.
                         * @return	null if it fails, else.
                         */
                        unsigned char * mPerformHashing (
                            const char * algName
                            , const unsigned char * data, int dataLen
                            , int & outBufSize )
                        throw ( Smartmatic::System::Exception::MessageDigestException );

                    private:

                        const Smartmatic::System::SafeBuffer & buffer; ///< buffer
                        static Smartmatic::Log::ISMTTLog * logger; ///< The logger
                };
            }
        }
    }
}

#endif /* MSGDIGESTUTIL_H_ */
