/**
 * @file XmlSigned.cxx
 * @brief Body class for XmlSigned file
 */

#include "XmlSigned.hxx"
#include <System/XML/DomXml.hxx>
#include <System/XML/ExtractCertificate.hxx>
#include <xsec/framework/XSECProvider.hpp>
#include <xsec/enc/OpenSSL/OpenSSLCryptoKeyRSA.hpp>
#include <xsec/enc/OpenSSL/OpenSSLCryptoKeyDSA.hpp>
#include <xsec/enc/OpenSSL/OpenSSLCryptoKeyEC.hpp>
#include <xsec/dsig/DSIGReference.hpp>
#include <xsec/dsig/DSIGKeyInfoX509.hpp>
#include "OpenSSLCertificate.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <memory>

#ifdef HAVE_CONFIG_H
#include "../../config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///< macro for gettext
#else
#define _(String) (String) ///< macro without gettext
#endif

#define N_(String) (String) ///< macro not translate

#define DSIGNS_PREFIX "" ///< prefix signed node

using namespace Smartmatic::System::Security;
using namespace Smartmatic::Log;

ISMTTLog * XmlSigned::logger = SMTTLogManager::GetLogger ( "Smartmatic.System.Security.XmlSigned", GETTEXT_PACKAGE );

XmlSigned::XmlSigned ( const Smartmatic::System::SafeBuffer & buffer )
    : buffer ( buffer )
{

}

XmlSigned::~XmlSigned()
{

}

bool XmlSigned::signBuffer ( EVP_PKEY * privateKey, Smartmatic::System::SafeBuffer & result, X509 * cert, hashMethod hm )
{
    OpenSSLCryptoX509 * openSSLCert = NULL;

    bool validate = false;
    result.Clear();

    if ( privateKey == NULL )
    {
        logger->Debug ( "ERROR - Private key" );
        return validate;
    }

    if ( buffer.GetSize() == 0 )
    {
        logger->Debug ( "ERROR - Xml Buffer" );
        return validate;
    }

    Smartmatic::System::XML::DomXml domXml;
    xercesc::DOMDocument * doc = domXml.bufferToDom ( buffer );

    try
    {
        signatureMethod sigMethod;
        XSECCryptoKey * key = getKey ( privateKey, sigMethod );

        if ( key == NULL )
        {
            logger->Debug ( "ERROR - Error key" );
            return validate;
        }

        XSECProvider prov;
        xercesc::DOMElement * rootElem = NULL;
        xercesc::DOMElement * sigNode = NULL;
        DSIGSignature * sig = NULL;

        rootElem = doc->getDocumentElement();

        if ( rootElem == NULL )
        {
            logger->Debug ( "ERROR - Xml format" );
            return validate;
        }

        sig = prov.newSignature();
        sig->setDSIGNSPrefix ( MAKE_UNICODE_STRING ( DSIGNS_PREFIX ) );
        sig->setPrettyPrint ( true );

        sigNode = sig->createBlankSignature ( doc, CANON_C14N_NOC, sigMethod, hm );
        rootElem->appendChild ( sigNode );
        rootElem->appendChild ( doc->createTextNode ( MAKE_UNICODE_STRING ( "\n" ) ) );

        if ( cert )
        {
            openSSLCert = new OpenSSLCryptoX509 ( cert );
        }

        if ( openSSLCert != NULL )
        {
            setKey ( sig, openSSLCert );
        }

        sig->setSigningKey ( key );
        sig->sign();

        prov.releaseSignature ( sig );

        validate = true;
        domXml.domToBuffer ( doc, result );
    }
    catch ( ... )
    {
        logger->Debug ( "ERROR - Signed" );
    }

    if ( openSSLCert )
    {
        delete openSSLCert;
        openSSLCert = NULL;
    }

    return validate;
}

bool XmlSigned::verifyBuffer ( X509 * cert )
{
    OpenSSLCryptoX509 * openSSLCert = NULL;
    bool validate = false;

    if ( buffer.GetSize() == 0 )
    {
        logger->Debug ( "ERROR - Xml Buffer" );
        return validate;
    }

    Smartmatic::System::XML::DomXml domXml;
    xercesc::DOMDocument * doc = domXml.bufferToDom ( buffer );

    Smartmatic::System::SafeBuffer base64Cert;
    Smartmatic::System::XML::ExtractCertificate extractCertificate ( buffer );
    extractCertificate.getBase64Certificate ( base64Cert );
    OpenSSLCertificate base64X509 ( base64Cert );

    XSECProvider prov;
    DSIGSignature * sig = NULL;

    try
    {
        OpenSSLCryptoX509* x509 = NULL;
        std::auto_ptr<OpenSSLCryptoX509> openSSLCert(new OpenSSLCryptoX509 ( cert ));

        if ( base64X509.getX509() != NULL )
        {
            x509 = base64X509.getX509();
        }
        else if ( cert != NULL )
        {
            x509 = openSSLCert.get();
        }
        else
        {
            logger->Debug ( "ERROR - Certificate" );
            return validate;
        }

        sig = prov.newSignatureFromDOM ( doc );

        sig->setSigningKey ( x509->clonePublicKey() );
        sig->load();

        validate = sig->verify();
    }
    catch ( ... )
    {
        logger->Debug ( "ERROR - Validate" );
    }

    if ( sig )
    {
        prov.releaseSignature ( sig );
    }

    if ( openSSLCert )
    {
        delete openSSLCert;
        openSSLCert = NULL;
    }

    return validate;
}

XSECCryptoKey * XmlSigned::getKey ( EVP_PKEY * primaryKey, signatureMethod & sigMethod )
{
    XSECCryptoKey * key = NULL;

    switch ( primaryKey->type )
    {
        case EVP_PKEY_RSA:
        case EVP_PKEY_RSA2:
            logger->Debug ( "Load Key RSA" );
            sigMethod = SIGNATURE_RSA;
            key = new OpenSSLCryptoKeyRSA ( primaryKey );
            break;

        case EVP_PKEY_DSA:
        case EVP_PKEY_DSA1:
        case EVP_PKEY_DSA2:
        case EVP_PKEY_DSA3:
        case EVP_PKEY_DSA4:
            logger->Debug ( "Load Key DSA" );
            sigMethod = SIGNATURE_DSA;
            key = new OpenSSLCryptoKeyDSA ( primaryKey );
            break;

        case EVP_PKEY_EC:
            logger->Debug ( "Load Key ECDSA" );
            sigMethod = SIGNATURE_ECDSA;
            key = new OpenSSLCryptoKeyEC ( primaryKey );
            break;

        default:
            sigMethod = SIGNATURE_NONE;
            break;
    }

    return key;
}

void XmlSigned::setKey ( void * sig, OpenSSLCryptoX509 * x509 )
{
    DSIGReference * ref = NULL;
    safeBuffer sb;
    const char * b64CertStr = NULL;
    XMLCh * b64Cert = NULL;
    DSIGKeyInfoX509 * keyInfoX509 = NULL;

    ref = ( ( DSIGSignature * ) sig )->createReference ( MAKE_UNICODE_STRING ( "" ),HASH_SHA256 );
    ( void ) ref->appendEnvelopedSignatureTransform();

    sb = x509->getDEREncodingSB();
    keyInfoX509 = ( ( DSIGSignature * ) sig )->appendX509Data();
    char buf[1024];
    X509_NAME_oneline ( X509_get_subject_name ( x509->getOpenSSLX509() ), buf, 1024 );
    std::string str = buf;

    keyInfoX509->setX509SubjectName ( MAKE_UNICODE_STRING ( str.c_str() ) );
    b64CertStr = sb.rawCharBuffer();
    b64Cert = XMLString::transcode ( b64CertStr );
    keyInfoX509->appendX509Certificate ( b64Cert );
    XSEC_RELEASE_XMLCH ( b64Cert );
}
