#ifndef TEST_PBE_KEY_DERIVATOR_TEST
#define TEST_PBE_KEY_DERIVATOR_TEST

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <System/Security/Cryptography/PBEKeyDerivatorHMAC.h>
#include <System/Security/OpenSSLSecurity.hxx>
#include <System/SafeBuffer.h>

class PBEKeyDerivatorHMACTest : public CPPUNIT_NS::TestFixture
{
    private:

        typedef Smartmatic::System::Security::Cryptography::PBEKeyDerivatorHMAC PBEKeyDerivatorHMAC;
        typedef Smartmatic::System::SafeBuffer SafeBuffer;

        PBEKeyDerivatorHMAC * manager;

    public:

        PBEKeyDerivatorHMACTest()
        {
            manager = NULL;
        }

        void setUp()
        {
            Smartmatic::System::Security::OpenSSLSecurity::init();
            manager = new PBEKeyDerivatorHMAC();
        }

        void tearDown()
        {
            Smartmatic::System::Security::OpenSSLSecurity::clear();

            if ( manager != NULL )
            {
                delete ( manager );
                manager = NULL;
            }
        }

        void testCreateKeyWithoutSaltEmptyBuffer()
        {
            SafeBuffer inKey;
            SafeBuffer outKey;

            manager->GenerateKey ( inKey, outKey, Smartmatic::System::Security::Cryptography::MsgDigestUtil::DGSTUTIL_SHA1 );

            CPPUNIT_ASSERT ( 0 == outKey.GetSize() );
        }

        void testCreateKeyWithoutSalt()
        {
            SafeBuffer inKey ( "ABCD" );
            SafeBuffer outKey;

            std::string expected = "3z2A/M2rn5895bIkKl9Kx/Gb8kT7R+fluRhApZpYdgY=";

            manager->GenerateKey ( inKey, outKey, Smartmatic::System::Security::Cryptography::MsgDigestUtil::DGSTUTIL_SHA1 );

            CPPUNIT_ASSERT ( 32 == outKey.GetSize() );
            CPPUNIT_ASSERT ( expected == outKey.GetBase64Data() );
        }

        void testCreateKeyWithoutSaltEmptyBufferTwoTime()
        {
            SafeBuffer inKey;
            SafeBuffer outKey;

            manager->GenerateKey ( inKey, outKey, Smartmatic::System::Security::Cryptography::MsgDigestUtil::DGSTUTIL_SHA1 );

            CPPUNIT_ASSERT ( 0 == outKey.GetSize() );

            manager->GenerateKey ( inKey, outKey, Smartmatic::System::Security::Cryptography::MsgDigestUtil::DGSTUTIL_SHA1 );

            CPPUNIT_ASSERT ( 0 == outKey.GetSize() );
        }

        void testCreateKeyWithoutSaltTwoTime()
        {
            SafeBuffer inKey ( "ABCD" );
            SafeBuffer outKey;

            std::string expected = "3z2A/M2rn5895bIkKl9Kx/Gb8kT7R+fluRhApZpYdgY=";

            manager->GenerateKey ( inKey, outKey, Smartmatic::System::Security::Cryptography::MsgDigestUtil::DGSTUTIL_SHA1 );

            CPPUNIT_ASSERT ( 32 == outKey.GetSize() );
            CPPUNIT_ASSERT ( expected == outKey.GetBase64Data() );

            manager->GenerateKey ( inKey, outKey, Smartmatic::System::Security::Cryptography::MsgDigestUtil::DGSTUTIL_SHA1 );

            CPPUNIT_ASSERT ( 32 == outKey.GetSize() );
            CPPUNIT_ASSERT ( expected == outKey.GetBase64Data() );
        }

        void testCreateKeyWithoutSaltKeyLenZero()
        {
            SafeBuffer inKey ( "ABCD" );
            SafeBuffer outKey;

            std::string expected = "3z2A/M2rn5895bIkKl9Kx/Gb8kT7R+fluRhApZpYdgY=";

            manager->GenerateKey ( inKey, outKey, Smartmatic::System::Security::Cryptography::MsgDigestUtil::DGSTUTIL_SHA1, 0 );

            CPPUNIT_ASSERT ( 32 == outKey.GetSize() );
            CPPUNIT_ASSERT ( expected == outKey.GetBase64Data() );
        }

        void testCreateKeyWithoutSaltKeyLen16()
        {
            SafeBuffer inKey ( "ABCD" );
            SafeBuffer outKey;

            std::string expected = "3z2A/M2rn5895bIkKl9Kxw==";

            manager->GenerateKey ( inKey, outKey, Smartmatic::System::Security::Cryptography::MsgDigestUtil::DGSTUTIL_SHA1, 16 );

            CPPUNIT_ASSERT ( 16 == outKey.GetSize() );
            CPPUNIT_ASSERT ( expected == outKey.GetBase64Data() );
        }

        void testCreateKeyWithoutSaltKeyLen8()
        {
            SafeBuffer inKey ( "ABCD" );
            SafeBuffer outKey;

            std::string expected = "3z2A/M2rn58=";

            manager->GenerateKey ( inKey, outKey, Smartmatic::System::Security::Cryptography::MsgDigestUtil::DGSTUTIL_SHA1, 8 );

            CPPUNIT_ASSERT ( 8 == outKey.GetSize() );
            CPPUNIT_ASSERT ( expected == outKey.GetBase64Data() );
        }

        void testCreateKeyWithSalt()
        {
            SafeBuffer inKey ( "ABCD" );
            SafeBuffer outKey;
            SafeBuffer salt ( "xyz" );

            std::string expected = "SiaOR6MJk38qDAtUi7hQd7EfT50VDOOAqpAATcBTFmA=";

            manager->SetSalt ( salt );
            manager->GenerateKey ( inKey, outKey, Smartmatic::System::Security::Cryptography::MsgDigestUtil::DGSTUTIL_SHA1 );

            CPPUNIT_ASSERT ( 32 == outKey.GetSize() );
            CPPUNIT_ASSERT ( expected == outKey.GetBase64Data() );
        }

        void testCreateKeySetSaltTwoTime()
        {
            SafeBuffer inKey ( "ABCD" );
            SafeBuffer outKey;
            SafeBuffer salt ( "xyz" );

            std::string expected = "SiaOR6MJk38qDAtUi7hQd7EfT50VDOOAqpAATcBTFmA=";

            manager->SetSalt ( salt );
            manager->SetSalt ( salt );
            manager->GenerateKey ( inKey, outKey, Smartmatic::System::Security::Cryptography::MsgDigestUtil::DGSTUTIL_SHA1 );

            CPPUNIT_ASSERT ( 32 == outKey.GetSize() );
            CPPUNIT_ASSERT ( expected == outKey.GetBase64Data() );
        }

        void testCreateKeyWithSaltTwoTime()
        {
            SafeBuffer inKey ( "ABCD" );
            SafeBuffer outKey;
            SafeBuffer salt ( "xyz" );

            std::string expected = "SiaOR6MJk38qDAtUi7hQd7EfT50VDOOAqpAATcBTFmA=";

            manager->SetSalt ( salt );
            manager->GenerateKey ( inKey, outKey, Smartmatic::System::Security::Cryptography::MsgDigestUtil::DGSTUTIL_SHA1 );

            CPPUNIT_ASSERT ( 32 == outKey.GetSize() );
            CPPUNIT_ASSERT ( expected == outKey.GetBase64Data() );

            manager->GenerateKey ( inKey, outKey, Smartmatic::System::Security::Cryptography::MsgDigestUtil::DGSTUTIL_SHA1 );

            CPPUNIT_ASSERT ( 32 == outKey.GetSize() );
            CPPUNIT_ASSERT ( expected == outKey.GetBase64Data() );
        }

        void testCreateKeyWithSaltAndIterationsZero()
        {
            SafeBuffer inKey ( "ABCD" );
            SafeBuffer outKey;
            SafeBuffer salt ( "xyz" );

            std::string expected = "t7XOqyZV8xqUEOefQre/SXWskm6mEiLtdXaIFKp5soA=";

            manager->SetSalt ( salt );
            manager->GenerateKey ( inKey, outKey, Smartmatic::System::Security::Cryptography::MsgDigestUtil::DGSTUTIL_SHA1, 32, 0 );

            CPPUNIT_ASSERT ( 32 == outKey.GetSize() );
            CPPUNIT_ASSERT ( expected == outKey.GetBase64Data() );
        }

        void testCreateKeyWithSaltAndIterations1024()
        {
            SafeBuffer inKey ( "ABCD" );
            SafeBuffer outKey;
            SafeBuffer salt ( "xyz" );

            std::string expected = "zEClf0FqoBPui5/4qQY3X5u/baLcK0LtU3h1ov6hy3c=";

            manager->SetSalt ( salt );
            manager->GenerateKey ( inKey, outKey, Smartmatic::System::Security::Cryptography::MsgDigestUtil::DGSTUTIL_SHA1, 32, 1024 );

            CPPUNIT_ASSERT ( 32 == outKey.GetSize() );
            CPPUNIT_ASSERT ( expected == outKey.GetBase64Data() );
        }

        void testCreateKeyWithSaltAndIterations1024AndMD5()
        {
            SafeBuffer inKey ( "ABCD" );
            SafeBuffer outKey;
            SafeBuffer salt ( "xyz" );

            std::string expected = "Z8a6E/EvwFUFWEss5EpSJ2+jWueWPZOpunuByVKo7Hs=";

            manager->SetSalt ( salt );
            manager->GenerateKey ( inKey, outKey, Smartmatic::System::Security::Cryptography::MsgDigestUtil::DGSTUTIL_MD5, 32, 1024 );

            CPPUNIT_ASSERT ( 32 == outKey.GetSize() );
            CPPUNIT_ASSERT ( expected == outKey.GetBase64Data() );
        }

        void testCreateKeyWithSaltAndIterations1024AndSHA1()
        {
            SafeBuffer inKey ( "ABCD" );
            SafeBuffer outKey;
            SafeBuffer salt ( "xyz" );

            std::string expected = "zEClf0FqoBPui5/4qQY3X5u/baLcK0LtU3h1ov6hy3c=";

            manager->SetSalt ( salt );
            manager->GenerateKey ( inKey, outKey, Smartmatic::System::Security::Cryptography::MsgDigestUtil::DGSTUTIL_SHA1, 32, 1024 );

            CPPUNIT_ASSERT ( 32 == outKey.GetSize() );
            CPPUNIT_ASSERT ( expected == outKey.GetBase64Data() );
        }

        void testCreateKeyWithSaltAndIterations1024AndSHA224()
        {
            SafeBuffer inKey ( "ABCD" );
            SafeBuffer outKey;
            SafeBuffer salt ( "xyz" );

            std::string expected = "XxUJarDzTrvLzJZuU7RDcbIMBvXBDQ0OvzsYpywELAU=";

            manager->SetSalt ( salt );
            manager->GenerateKey ( inKey, outKey, Smartmatic::System::Security::Cryptography::MsgDigestUtil::DGSTUTIL_SHA224, 32, 1024 );

            CPPUNIT_ASSERT ( 32 == outKey.GetSize() );
            CPPUNIT_ASSERT ( expected == outKey.GetBase64Data() );
        }

        void testCreateKeyWithSaltAndIterations1024AndSHA256()
        {
            SafeBuffer inKey ( "ABCD" );
            SafeBuffer outKey;
            SafeBuffer salt ( "xyz" );

            std::string expected = "VDv+wZXb8+oO271a+5LYwGhfJMjYVmi3rnM8Z8nk+5U=";

            manager->SetSalt ( salt );
            manager->GenerateKey ( inKey, outKey, Smartmatic::System::Security::Cryptography::MsgDigestUtil::DGSTUTIL_SHA256, 32, 1024 );

            CPPUNIT_ASSERT ( 32 == outKey.GetSize() );
            CPPUNIT_ASSERT ( expected == outKey.GetBase64Data() );
        }

        void testCreateKeyWithSaltAndIterations1024AndSHA384()
        {
            SafeBuffer inKey ( "ABCD" );
            SafeBuffer outKey;
            SafeBuffer salt ( "xyz" );

            std::string expected = "qafPgYIFVEu2+S0YagbuNMG2dFn2xCx7Tbo7MfuSXyc=";

            manager->SetSalt ( salt );
            manager->GenerateKey ( inKey, outKey, Smartmatic::System::Security::Cryptography::MsgDigestUtil::DGSTUTIL_SHA384, 32, 1024 );

            CPPUNIT_ASSERT ( 32 == outKey.GetSize() );
            CPPUNIT_ASSERT ( expected == outKey.GetBase64Data() );
        }

        void testCreateKeyWithSaltAndIterations1024AndSHA512()
        {
            SafeBuffer inKey ( "ABCD" );
            SafeBuffer outKey;
            SafeBuffer salt ( "xyz" );

            std::string expected = "wUzIS8GIdfXe/n9fLtmDg4zFwzomP2YCEQjjmQfqLic=";

            manager->SetSalt ( salt );
            manager->GenerateKey ( inKey, outKey, Smartmatic::System::Security::Cryptography::MsgDigestUtil::DGSTUTIL_SHA512, 32, 1024 );

            CPPUNIT_ASSERT ( 32 == outKey.GetSize() );
            CPPUNIT_ASSERT ( expected == outKey.GetBase64Data() );
        }

        CPPUNIT_TEST_SUITE ( PBEKeyDerivatorHMACTest );

        CPPUNIT_TEST ( testCreateKeyWithoutSaltEmptyBuffer );
        CPPUNIT_TEST ( testCreateKeyWithoutSalt );
        CPPUNIT_TEST ( testCreateKeyWithoutSaltEmptyBufferTwoTime );
        CPPUNIT_TEST ( testCreateKeyWithoutSaltTwoTime );

        CPPUNIT_TEST ( testCreateKeyWithoutSaltKeyLenZero );
        CPPUNIT_TEST ( testCreateKeyWithoutSaltKeyLen16 );
        CPPUNIT_TEST ( testCreateKeyWithoutSaltKeyLen8 );

        CPPUNIT_TEST ( testCreateKeyWithSalt );
        CPPUNIT_TEST ( testCreateKeySetSaltTwoTime );
        CPPUNIT_TEST ( testCreateKeyWithSaltTwoTime );

        CPPUNIT_TEST ( testCreateKeyWithSaltAndIterationsZero );
        CPPUNIT_TEST ( testCreateKeyWithSaltAndIterations1024 );

        CPPUNIT_TEST ( testCreateKeyWithSaltAndIterations1024AndMD5 );
        CPPUNIT_TEST ( testCreateKeyWithSaltAndIterations1024AndSHA1 );
        CPPUNIT_TEST ( testCreateKeyWithSaltAndIterations1024AndSHA224 );
        CPPUNIT_TEST ( testCreateKeyWithSaltAndIterations1024AndSHA256 );
        CPPUNIT_TEST ( testCreateKeyWithSaltAndIterations1024AndSHA384 );
        CPPUNIT_TEST ( testCreateKeyWithSaltAndIterations1024AndSHA512 );

        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_PBE_KEY_DERIVATOR_TEST */
