/**
 * PBEFileUtil.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.security.asymmetric;

import java.io.File;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

import javax.crypto.BadPaddingException;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.SecretKey;

import org.apache.commons.io.FileUtils;
import org.bouncycastle.crypto.generators.PKCS5S2ParametersGenerator;
import org.bouncycastle.crypto.params.KeyParameter;

import com.smartmatic.waffle.security.dto.CipherDTO;
import com.smartmatic.waffle.security.lang.CryptographicRuntimeException;
import com.smartmatic.waffle.security.lang.CryptographicStaticException;
import com.smartmatic.waffle.security.symmetric.PoolableSymmetricCipher;
import com.smartmatic.waffle.security.symmetric.SymmetricKeyManager;

/**
 * Class that handles input/output operations to and from files ciphered using
 * password-based encryption (PBE).
 */

public class PBEFileUtil {

	/**
	 * Enum that holds the algorithm types suported to ciphers data.
	 */
	public enum CipheringAlgorithm {
		CBC, ECB;
	}

	/**
	 * Reads a file ciphered using PBE and returns the data contained in the
	 * file as a byte array.
	 * 
	 * @param sourceFile
	 *            The file to read
	 * @param password
	 *            The password used to cipher the file using PBE.
	 * 
	 * @param cipheringAlgorithm
	 *            Indicates the algorithm used to cipher the files (CBC or ECB)
	 * 
	 * @param iVectorLength
	 *            Length of initialization vector for cbc ciphering
	 * 
	 * @return A byte array with the file content
	 * 
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 * @throws CryptographicStaticException
	 *             the cryptographic static exception
	 * @throws CryptographicRuntimeException
	 *             the cryptographic runtime exception
	 * @throws IllegalBlockSizeException
	 *             the illegal block size exception
	 * @throws BadPaddingException
	 *             the bad padding exception
	 * @throws NoSuchAlgorithmException
	 *             the no such algorithm exception
	 */

	public static byte[] readDataFromCipheredFile(File sourceFile,
			String password, CipheringAlgorithm cipheringAlgorithm,
			Integer iVectorLength, String saltSeed) throws IOException,
			CryptographicStaticException, CryptographicRuntimeException,
			IllegalBlockSizeException, BadPaddingException,
			NoSuchAlgorithmException {

		byte[] plainData = null;

		byte[] rawFileData = FileUtils.readFileToByteArray(sourceFile);

		// The file will contain only the ciphered data

		byte[] salt = generateSalt(saltSeed);
		byte[] key = generatePbeKey(salt, 2048, password);

		SecretKey secretKey = SymmetricKeyManager.getSymmetricKey(key);

		PoolableSymmetricCipher symmetricCipher = null;

		switch (cipheringAlgorithm) {
		case CBC: {

			CipherDTO cipherDTO = EncryptDecryptUtil.prepareCipherDTO(
					iVectorLength, rawFileData, 0);

			symmetricCipher = new PoolableSymmetricCipher(secretKey,
					cipherDTO.getiVector());
			plainData = symmetricCipher.decryptBytes(cipherDTO
					.getCipheredData());
		}
			break;

		case ECB:
			symmetricCipher = new PoolableSymmetricCipher(secretKey);
			plainData = symmetricCipher.decryptBytes(rawFileData);
			break;
		}

		return plainData;
	}

	/**
	 * Writes a file and ciphers the file using PBE. This version only saves the
	 * data (without salt and iteration count)
	 * 
	 * @param targetFile
	 *            The file where to write
	 * @param password
	 *            the password for ciphering the data
	 * @param data
	 *            The data to write in the file
	 * @param cipheringAlgorithm
	 *            Indicates the algorithm used to cipher the files (CBC or ECB)
	 * @param iVectorLength
	 *            Length of vector for cbc ciphering
	 * 
	 * @throws CryptographicRuntimeException
	 *             the cryptographic runtime exception
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 * @throws IllegalBlockSizeException
	 *             the illegal block size exception
	 * @throws BadPaddingException
	 *             the bad padding exception
	 * @throws NoSuchAlgorithmException
	 *             the no such algorithm exception
	 * @throws CryptographicStaticException
	 *             the cryptographic static exception
	 */

	public static void writeDataToCipheredFile(File targetFile,
			String password, byte[] data, String saltSeed,
			CipheringAlgorithm cipheringAlgorithm)
			throws CryptographicRuntimeException, IOException,
			IllegalBlockSizeException, BadPaddingException,
			NoSuchAlgorithmException, CryptographicStaticException {

		byte[] encData = cipherData(password, data, saltSeed,
				cipheringAlgorithm);

		FileUtils.writeByteArrayToFile(targetFile, encData);
	}

	/**
	 * Ciphers data using a password and cipher algorithm
	 * 
	 * @param password
	 *            the password for ciphering the data
	 * @param data
	 *            The data to write in the file
	 * @param saltSeed
	 *            seed to create the salt
	 * @param cipheringAlgorithm
	 *            Indicates the algorithm used to cipher the files (CBC or ECB)
	 * @return the ciphered data
	 * @throws NoSuchAlgorithmException
	 *             the no such algorithm exception
	 * @throws UnsupportedEncodingException
	 *             the unsupported encoding exception
	 * @throws CryptographicStaticException
	 *             the cryptographic static exception
	 * @throws CryptographicRuntimeException
	 *             the cryptographic runtime exception
	 */
	public static byte[] cipherData(String password, byte[] data,
			String saltSeed, CipheringAlgorithm cipheringAlgorithm)
			throws NoSuchAlgorithmException, UnsupportedEncodingException,
			CryptographicStaticException, CryptographicRuntimeException {

		byte[] salt = generateSalt(saltSeed);

		SecretKey secretKey = SymmetricKeyManager
				.getSymmetricKey(generatePbeKey(salt, 2048, password));

		byte[] encData = null;

		switch (cipheringAlgorithm) {
		case CBC: {

			byte[] iVector = SymmetricKeyManager.generateIVectorByte();

			encData = EncryptDecryptUtil.encryptData(data, secretKey, iVector);
		}
			break;

		case ECB:
			encData = EncryptDecryptUtil.encryptData(data, secretKey);
		}

		return encData;
	}

	/**
	 * Generates the PBE key.
	 * 
	 * @param salt
	 *            the salt
	 * @param iterationCount
	 *            the iteration count
	 * @param password
	 *            the password
	 * @return the byte[]
	 * @throws UnsupportedEncodingException
	 *             the unsupported encoding exception
	 */
	private static byte[] generatePbeKey(byte[] salt, int iterationCount,
			String password) throws UnsupportedEncodingException {

		PKCS5S2ParametersGenerator parametersGenerator = new PKCS5S2ParametersGenerator();

		parametersGenerator.init(password.getBytes("UTF-8"), salt,
				iterationCount);
		KeyParameter generateDerivedParameters = (KeyParameter) parametersGenerator
				.generateDerivedParameters(256);

		byte[] key = generateDerivedParameters.getKey();

		return key;
	}

	/**
	 * Generates the salt to cipher data.
	 * 
	 * @param password
	 *            the password
	 * @return the byte[]
	 * @throws NoSuchAlgorithmException
	 *             the no such algorithm exception
	 * @throws UnsupportedEncodingException
	 *             the unsupported encoding exception
	 */
	private static byte[] generateSalt(String password)
			throws NoSuchAlgorithmException, UnsupportedEncodingException {

		MessageDigest digest = MessageDigest.getInstance("SHA-256");
		byte[] data = digest.digest(password.getBytes("UTF-8"));

		return data;

	}
}
