/*
 * DetachedURIDereferencer.java
 *
 * Copyright (c) 2012 Smartmatic Intl.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Intl. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Intl.
 */
package com.smartmatic.waffle.security.sign;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Collections;
import java.util.Set;

import javax.xml.crypto.Data;
import javax.xml.crypto.OctetStreamData;
import javax.xml.crypto.URIDereferencer;
import javax.xml.crypto.URIReference;
import javax.xml.crypto.URIReferenceException;
import javax.xml.crypto.XMLCryptoContext;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;

/**
 * This class is an implementation of an {@link URIDereferencer} and contains
 * the logic needed to resolve the relative paths used in the files references
 * contained in a detached signatures file. It is useful to sign different sets
 * of files with different base URI as if they have the same base URI.
 * 
 * @version $Revision:$ ($Date:$)
 * @author Jose Carrizo (jose.carrizo@smartmatic.com)
 */
public class DetachedURIDereferencer implements URIDereferencer {

	/**
	 * Set of base URIs
	 */
	private Set<String> baseURIs;

	/**
	 * Constructor of DetachedURIDereferencer
	 * 
	 * @param theBaseURIs
	 *            Set of base URIs were are located the files to be signed
	 */
	public DetachedURIDereferencer(Set<String> theBaseURIs) {
		if (theBaseURIs == null) {
			this.baseURIs = Collections.emptySet();
		} else {
			this.baseURIs = theBaseURIs;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * javax.xml.crypto.URIDereferencer#dereference(javax.xml.crypto.URIReference
	 * , javax.xml.crypto.XMLCryptoContext)
	 */
	@Override
	public Data dereference(URIReference uriReference, XMLCryptoContext context)
			throws URIReferenceException {

		for (String uri : this.baseURIs) {
			File file = new File(uri, uriReference.getURI());
			if (file.exists()) {
				return getFileData(file);
			}
		}

		File file = new File(uriReference.getURI());
		if (file.exists()) {
			return getFileData(file);
		}

		return null;
	}

	/**
	 * Gets the data of the given file
	 * 
	 * @param theFile
	 *            The file
	 * @throws URIReferenceException
	 */
	private static Data getFileData(File theFile) throws URIReferenceException {
		ByteArrayInputStream byteArrayInputStream = null;
		try {
			byteArrayInputStream = new ByteArrayInputStream(
					FileUtils.readFileToByteArray(theFile));
			OctetStreamData data = new OctetStreamData(byteArrayInputStream);
			return data;
		} catch (FileNotFoundException e) {
			throw new URIReferenceException(e);
		} catch (IOException e) {
			throw new URIReferenceException(e);
		} finally {
			IOUtils.closeQuietly(byteArrayInputStream);
		}
	}
}
