/**
 * PoolablePBESymmetricCipher.java
 *
 * $Revision: 1.5 $ ($Date: 2011-04-06 04:47:28 $)
 *
 * Copyright (c) 2004 Smartmatic Intl.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Intl. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Intl.
 */
package com.smartmatic.waffle.security.symmetric;

import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.spec.InvalidKeySpecException;

import javax.crypto.Cipher;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.SecretKeyFactory;
import javax.crypto.spec.PBEKeySpec;

import com.smartmatic.waffle.security.lang.CryptographicRuntimeException;
import com.smartmatic.waffle.security.lang.CryptographicStaticException;

/**
 * <p>
 * Description: Class used to encrypt and decrypt information using a symmetric
 * algorithm. In the methods that this class receives or returns a encrypted
 * String is supposed that the String is encoded in Base 64.
 * </p>
 * <p>
 * This class interacts with all the cryptographic objects using the Java
 * Cryptographic Extension (JCE). Using the JCE specification the change of the
 * cryptographic algorithm is done changing the name of the algorithms
 * </p>
 * <p>
 * This class always returns and expects printable strings. In the encryption
 * process after the codification of the information, the result byte array is
 * converted into a Base64 String. In the decryption process before START the
 * decryption process this class convert the Base64 String into a byte array
 * that is passed to the cipher to decrypt.
 * </p>
 * 
 * Used PBE Symmetric cryptography standard
 * 
 * @author Carlos Lameda (carlos.lameda@smartmatic.com)
 * @version 1.0 (Nov 5, 2009)
 */
public class PoolablePBESymmetricCipher extends PoolableSymmetricCipher {

	/**
	 * Extended name of the algorithm that will be used in the encryption /
	 * decryption process. The format to define the extended name of the
	 * algorithm is: Algorithm/Mode/Padding. This is the extended algorithm
	 * definition used when the Cipher Block chaining is used as output cipher
	 * mode </p>
	 * 
	 * @see <a href=
	 *      "http://java.sun.com/j2se/1.4.2/docs/guide/security/jce/JCERefGuide.html#CipherClass"
	 *      > Java Cryptographic Extension< /a>
	 */
	private static final String ALGORITHM_PBE = "PBEWithSHAAnd3KeyTripleDES";

	// private static final String ALGORITHM_PBE = "PBEWithSHAAndTwofish-CBC";

	/**
	 * Symmetric PBE Key random bits
	 */
	private byte[] salt;

	/**
	 * Symmetric PBE Key iterations
	 */
	private int iterationCount;

	/**
	 * Getter for the <code>salt</code> attribute.
	 * 
	 * @return <code>salt</code> attribute value.
	 * @see PoolablePBESymmetricCipher#salt
	 */
	public byte[] getSalt() {
		return this.salt;
	}

	/**
	 * Setter for the <code>salt</code> attribute.
	 * 
	 * @param theSalt
	 *            the salt to set
	 * @see PoolablePBESymmetricCipher#salt
	 */
	public void setSalt(byte[] theSalt) {
		this.salt = (theSalt != null) ? theSalt.clone() : null;
	}

	/**
	 * Getter for the <code>iterationCount</code> attribute.
	 * 
	 * @return <code>iterationCount</code> attribute value.
	 * @see PoolablePBESymmetricCipher#iterationCount
	 */
	public int getIterationCount() {
		return this.iterationCount;
	}

	/**
	 * Setter for the <code>iterationCount</code> attribute.
	 * 
	 * @param iterationCount
	 *            the iterationCount to set
	 * @see PoolablePBESymmetricCipher#iterationCount
	 */
	public void setIterationCount(int iterationCount) {
		this.iterationCount = iterationCount;
	}

	/**
	 * Default class constructor
	 * 
	 * @param passphrase
	 *            the password
	 * @param theSalt
	 *            The salt
	 * @param iterationCount
	 *            Symmetric PBE Key iterations
	 * @throws CryptographicStaticException
	 *             thrown when exist a configuration error in the system
	 * @throws CryptographicRuntimeException
	 *             thrown when exist a problem with related to cryptographic
	 *             configuration as keys
	 */
	public PoolablePBESymmetricCipher(char[] passphrase, byte[] theSalt,
			int iterationCount) throws CryptographicStaticException,
			CryptographicRuntimeException {
		try {

			// Load the general parameters

			this.salt = (theSalt != null) ? theSalt.clone() : null;
			this.iterationCount = iterationCount;

			// Generate the PBE key

			PBEKeySpec keySpec = new PBEKeySpec(passphrase, theSalt,
					iterationCount);
			SecretKeyFactory keyFactory = SecretKeyFactory
					.getInstance(PoolablePBESymmetricCipher.ALGORITHM_PBE);

			// Generate the cipher

			this.decodingCipher = Cipher
					.getInstance(PoolablePBESymmetricCipher.ALGORITHM_PBE);
			this.encodingCipher = Cipher
					.getInstance(PoolablePBESymmetricCipher.ALGORITHM_PBE);

			this.secretKey = keyFactory.generateSecret(keySpec);

			this.encodingCipher.init(Cipher.ENCRYPT_MODE, this.secretKey);
			this.decodingCipher.init(Cipher.DECRYPT_MODE, this.secretKey);
		} catch (NoSuchPaddingException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.PADDING_OR_PROVIDER_PROBLEM,
					e);
		} catch (NoSuchAlgorithmException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.ALGORITHM_OR_PROVIDER_PROBLEM,
					e);
		} catch (InvalidKeyException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.SYMMETRIC_KEY_PROBLEM,
					e);
		} catch (InvalidKeySpecException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.SYMMETRIC_KEY_PROBLEM,
					e);
		}
	}
}
